#define TASK_COMM_LEN 16
#define KERN_STACKID_FLAGS (0 | BPF_F_FAST_STACK_CMP)
#define SOFTIRQ_TYPE 1
#define HARDIQR_TYPE 2
#define SPECTIAL_VECTOR_TYPE 3


/*copy from kernel*/
/*
 * Linux IRQ vector layout.
 *
 * There are 256 IDT entries (per CPU - each entry is 8 bytes) which can
 * be defined by Linux. They are used as a jump table by the CPU when a
 * given vector is triggered - by a CPU-external, CPU-internal or
 * software-triggered event.
 *
 * Linux sets the kernel code address each entry jumps to early during
 * bootup, and never changes them. This is the general layout of the
 * IDT entries:
 *
 *  Vectors   0 ...  31 : system traps and exceptions - hardcoded events
 *  Vectors  32 ... 127 : device interrupts
 *  Vector  128         : legacy int80 syscall interface
 *  Vectors 129 ... LOCAL_TIMER_VECTOR-1
 *  Vectors LOCAL_TIMER_VECTOR ... 255 : special interrupts
 *
 * 64-bit x86 has per CPU IDT tables, 32-bit has one shared IDT table.
 *
 * This file enumerates the exact layout of them:
 */
#define ERROR_APIC_VECTOR               0xfe
#define RESCHEDULE_VECTOR               0xfd
#define CALL_FUNCTION_VECTOR            0xfc
#define CALL_FUNCTION_SINGLE_VECTOR     0xfb
#define THERMAL_APIC_VECTOR             0xfa
#define THRESHOLD_APIC_VECTOR           0xf9
#define REBOOT_VECTOR                   0xf8
#define LOCAL_TIMER_VECTOR              0xec
#define NR_VECTORS                       256

struct bpf_arg
{
    __u64 min_n;
    bool stack;
};


struct irq_enter
{
    __u64 enter_time;
    __u64 utime;
    __u64 stime;
    __u64 sum_exec_runtime;
};


struct event
{
    int event_type;
    char task[TASK_COMM_LEN];
    __u32 pid;
    __u32 tgid;
    __u32 cpu;
    __u64 this_utime;
    __u64 this_stime;
    __u64 exec_time;
    int k_stack_id;
    int u_stack_id;
    __u64 irq_runtime;
    __u32 irq_num;
    // 用于 hardirq 输出
    char hi_name[32];
};
